@@ -13,7 +13,8 @@ var.mgm <- function(
   method = 'glm',  # which method should be used for each nodewise regression?
   missings = 'error', # handling of missing data
   weights = NA, # weights for observations 
-  ret.warn = TRUE # TRUE returns warnings, makes sense to switch off for time varying wrapper
+  ret.warn = TRUE, # TRUE returns warnings, makes sense to switch off for time varying wrapper
+  binary.sign = FALSE # see help file
 )
 
 {
@@ -31,9 +32,11 @@ var.mgm <- function(
                          missings = missings, 
                          weights = weights, 
                          ret.warn = ret.warn, 
+                         binary.sign = binary.sign,
                          VAR = TRUE) # use standard mgm.fit; no AR model
-
-  class(outlist) <- c('mgm', 'var')
+  
+  # Return estimation messages:
+  estimation_msg('var.mgm') # note about where signs are stored
   
   return(outlist)
 } 
